#-----------------------------------------------------------------------------
# Pick Random Poses  (follower for One Pose Per Frame)
# for Semidieu
# Copyright @ 2004, David G. Drumright (ockham)
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser
import sys
import os
import random
random.seed()
scene = poser.Scene()
BasePathUncut=poser.AppLocation()
BasePath=BasePathUncut[:(-len('poser.exe'))]
PZ2Path=os.path.abspath(BasePath+'/runtime/libraries/pose/')

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Randomize poses.")
        self.BaseName=''

        self.StatusEntry = Entry(self.master,width=40)
        self.StatusEntry.grid(row=0,column=0,columnspan=4)
        self.StatusEntry.insert(0,"First, hit Name to pick one file in the set.")

        # Go/nogo buttons.
        self.buttonName = Button(self.master, text="Name", command=self.HandleName)
        self.buttonName.grid(row=3, column=0)

        self.buttonGo = Button(self.master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=1)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


    # - - - - - - - - - - - - - - - - - -
    def FakeFrames(self,DesiredCount):
        # Write a short PZ2 file forcing the desired number of
        # frames; load it; delete it.
        PZ2Name=os.path.normpath(PZ2Path+'/zzztemp.pz2')
        fpFake=open(PZ2Name,"wt")
        if not fpFake: return 0
        fpFake.write('{\nversion\n\t{\n\tnumber 4.2\n\t}\nmovieInfo\n\t{\n\t')
        fpFake.write('numFrames %d\n\t}\n}' % DesiredCount)
        fpFake.close()
        try:
            scene.LoadLibraryPose(PZ2Name)
        except:
            return 0
        #os.remove(PZ2Name)
        return DesiredCount

    # - - - - - - - - - - - - - - - - - -
    def HandleName(self):
        fn=tkFileDialog.askopenfilename(initialfile=PZ2Path, filetypes=[("Pose files", "*.PZ2 *.pz2")] )

        if not fn:
            self.ShowStatus("No name selected. Try again.")
            return 0

        self.BaseName=os.path.normpath(fn)

        self.ShowStatus("Now hit Go to apply the numbered set at random.")

    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        GoodNames=[]
        # At this point, self.Basename is the full pathname of one member of the set.
        # We first get rid of PZ2:
        MainName,ext=os.path.splitext(self.BaseName)
        # Isolate the non-numbered part of the name so we can gather all the numbered parts.
        Parts=string.split(MainName,"_")
        # Find how many we have:
        for Index in range(1,999):
            # Build a candidate name
            TrialName="%s_%.3d.pz2" % (Parts[0],Index)
            if os.path.exists(TrialName):
                GoodNames.append(TrialName)

        # Now count the frames and add if necessary:
        FrameCount=len(GoodNames)+1
        if FrameCount < 1:
            self.ShowStatus("Problem with locating pose files.")
            return
        elif FrameCount > scene.NumFrames():
            self.ShowStatus("Adding frames to reach %s" % FrameCount)
            self.FakeFrames(FrameCount)

        # Now load the names at random.  The remove action insures no repetition.
        Index=0
        while len(GoodNames):
            OneName = random.choice(GoodNames)
            scene.SetFrame(Index)
            self.ShowStatus("Setting random pose at frame %d" % (Index+1))
            scene.LoadLibraryPose(OneName)
            GoodNames.remove(OneName)
            Index=Index+1

        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -
    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents(1)
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
